export default {
    Add: 'plus',
    Edit: 'pencil',
    Delete: 'delete',
    Success: 'check',
    Error: 'alert-circle',
    Pending: 'sync',
    GpsStart: 'crosshairs-gps',
    GpsStop: 'crosshairs-off',
    Search: 'magnify'
};
