export default {
    // Index
    OTHER_PAGES: 'Options',
    MODEL_PAGES: 'Content',

    // AutoForm, Login
    CANCEL: 'Cancel',
    SUBMIT: 'Submit',

    // DefaultList
    LIST_IS_EMPTY: 'Empty list.',
    SYNCED_ITEMS: 'Synced Items',

    // DeleteForm
    CONFIRM_DELETE: 'Are you sure you want to delete this record?',

    // DeleteForm.native
    CONFIRM_DELETE_TITLE: 'Confirm Deletion',
    CONFIRM_DELETE_OK: 'Yes, Delete',
    CONFIRM_DELETE_CANCEL: 'Cancel',

    // OutboxList
    EMPTY_OUTBOX: 'Empty Outbox',
    RETRY_ALL: 'Retry All',
    OUTBOX_IS_EMPTY: 'No items in outbox.',
    SYNC_ERROR: 'One or more errors found.',
    SYNC_SUCCESS: 'Successfully synced.',
    UNSYNCED_ITEMS: 'Unsynced Items',

    // Logout
    LOGGED_OUT: 'Logged out.',
    LOGGING_OUT: 'Logging out...',

    // Loading
    LOADING: 'Loading...',

    // Geo
    GEO_POINT_NEW: 'Click the {TOOL_ICON} tool to draw a new point.',
    GEO_POINT_EDIT:
        'Select the existing point to move it, or click the {TOOL_ICON} tool to draw a new point.',
    GEO_LINE_STRING_NEW: 'Click the {TOOL_ICON} tool to draw a new line.',
    GEO_LINE_STRING_EDIT:
        'Select the existing line to move it or its vertices.',
    GEO_POLYGON_NEW: 'Click the {TOOL_ICON} tool to draw a new polygon.',
    GEO_POLYGON_EDIT: 'Select the existing polygon to move it or its vertices.'
};
