import Add from '@material-ui/icons/Add';
import Edit from '@material-ui/icons/Edit';
import Delete from '@material-ui/icons/Delete';
import Success from '@material-ui/icons/Done';
import Error from '@material-ui/icons/Error';
import Pending from '@material-ui/icons/Sync';
import Expand from '@material-ui/icons/ExpandMore';
import Collapse from '@material-ui/icons/ExpandLess';
import GpsStart from '@material-ui/icons/GpsFixed';
import GpsStop from '@material-ui/icons/GpsOff';
import Search from '@material-ui/icons/Search';

export {
    Add,
    Edit,
    Delete,
    Success,
    Error,
    Pending,
    Expand,
    Collapse,
    GpsStart,
    GpsStop,
    Search
};
