import React from 'react';

export default function FooterContent() {
    return (
        <>
            Powered by{' '}
            <a href="https://wq.io/">
                <b>wq</b>
            </a>
        </>
    );
}
