export function Add() {
    return '\u2795'; // Plus
}

export function Edit() {
    return '\u270f'; // Pencil
}

export function Delete() {
    return '\u{1f5d1}'; // Wastebasket
}

export function Success() {
    return '\u2714'; // Check Mark
}

export function Error() {
    return '\u274c'; // Cross Mark
}

export function Pending() {
    return '\u23f3'; // Hourglass Not Done
}

export function GpsStart() {
    return '\u{1f6f0}\u{fe0f}'; // Satellite
}

export function GpsStop() {
    return '\u{1f6d1}'; // Stop Sign
}

export function Search() {
    return '\u{1f50d}'; // Magnifying Glass
}
